/*
	DoubleCheck, a UCI chess playing engine. Copyright (C) 2011 Lucas Braesch.

	DoubleCheck is free software: you can redistribute it and/or modify it under the terms of the GNU
	General Public License as published by the Free Software Foundation, either version 3 of the
	License, or (at your option) any later version.

	DoubleCheck is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
	even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License along with this program.
	If not, see <http://www.gnu.org/licenses/>.
*/
#include "test.h"

void test_movegen()
{
	assert(BitboardInitialized && PSQInitialized && ZobristInitialized);
	const int fen_count = 4;
	const int fen_depth[] = {5, 4, 6, 5};
	const char *fen[] = {
		"rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq -",
		"r3k2r/p1ppqpb1/bn2pnp1/3PN3/1p2P3/2N2Q1p/PPPBBPPP/R3K2R w KQkq -",
		"8/2p5/3p4/KP5r/1R3p1k/8/4P1P1/8 w - -",
		"r2q1rk1/pP1p2pp/Q4n2/bbp1p3/Np6/1B3NBn/pPPP1PPP/R3K2R b KQ -"
	};
	const U64 perft_correct[] = {4865609, 4085603, 11030083, 15833292};
	Board B;

	for (int i = 0; i < fen_count; i++) {
		load_fen(&B, fen[i]);
		printf("fen = %s\n", fen[i]);
		print_board(&B);
		U64 result = perft(&B, fen_depth[i], 0);

		if (perft_correct[i] != result) {
			fprintf(stderr, "=> incorrect perft(%d) for fen \"%s\"!\n", fen_depth[i], fen[i]);
			exit(EXIT_FAILURE);
		}
	}

	puts("=> move generator OK");
}

void test_search(const char *epd, int time_ms, int ttable_mb)
{
	assert(BitboardInitialized && PSQInitialized && ZobristInitialized);
	Board B;

	FILE *f = fopen(epd, "r");
	if (!f) {
		printf("file \"%s\" not found!\n", epd);
		exit(EXIT_FAILURE);
	}

	tt_alloc(ttable_mb);
	char fen[256], s[16];

	while (!feof(f)) {
		if (!fgets(fen, 256, f)) break;
		printf("=> searching \"%s\" for %f seconds\n", fen, time_ms/1000.0f);
		*strstr(fen, "bm") = '\0';	// cut the fen

		load_fen(&B, fen);
		print_board(&B);

		move_t m = id_loop(&B, time_ms, 0, 0);
		move_to_string(&B, m, s, true);
		printf("%s bm %s\n", fen, s);
	}

	tt_free();
	fclose(f);
}